@@ -21,8 +21,8 @@
 
 ;; You should have received a copy of the GNU General Public License
 ;; along with AUCTeX; see the file COPYING.  If not, write to the Free
-;; Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-;; 02111-1307, USA.
+;; Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+;; 02110-1301, USA.
 
 ;;; Commentary:
 ;;
@@ -83,6 +83,9 @@
 (unless (get 'TeX-source-specials-view-editor-flags 'saved-value)
   (setq TeX-source-specials-view-editor-flags ""))
 
+(unless (get 'TeX-kpathsea-path-delimiter 'saved-value)
+  (setq TeX-kpathsea-path-delimiter ";"))
+
 (provide 'tex-fptex)
 (require 'tex-site)
 
