@@ -1,9 +1,13 @@
 ;; ess-latex.el - Interface ESS and LaTeX.
+
 ;; Copyright (C) 1999 by A.J. Rossini <rossini@biostat.washington.edu>
 ;; based on noweb-mode by: Thorsten.Ohl @ Physik.TH-Darmstadt.de
 ;;     with a little help from Norman Ramsey <norman@bellcore.com>
 ;;                         and Mark Lunt <mark.lunt@mrc-bsu.cam.ac.uk>
-;;                         and A.J. Rossini <rossini@biostat.washington.edu>
+
+;; Maintainers: ESS-core <ESS-core@stat.math.ethz.ch>
+
+;; This file is part of ESS.
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -21,20 +25,16 @@
 ;; 
 ;; See bottom of this file for information on language-dependent
 ;; highlighting, and recent changes.
-;;
 
-;; BASED ON: (from Mark Lunt).
-;; ESS CVS: --Id: noweb-mode.el,v 1.1 1999/04/21 18:03:51 rossini Exp --
 
 
-
 ;;; Variables
 
 (defconst ess-latex-minor-mode-RCS-Id
-  "$Id: ess-latex.el,v 1.2 2004/07/02 14:46:35 rsparapa Exp $")
+  "Imported to ESS Subversion repository and RCS ids not maintained.")
 
 (defconst ess-latex-minor-mode-RCS-Name
-  "$Name: sumo-2004-08-18 $")
+  " ")
 
 (defvar ess-latex-minor-mode-prefix "\M-n"
   "*Prefix key to use for noweb mode commands.
@@ -484,14 +484,15 @@ otherwise."
              (setq mode (concat                                            
                          (downcase (buffer-substring beg end))             
                          "-mode"))
-             (if (and (>= (length mode) 11)
+             (if ;(and 
+		 (>= (length mode) 11)
                       (progn
                         (if
                             (equal (substring mode -10 -5) "-mode")
                             (setq mode (substring mode 0 -5)))
                         (if 
                             (equal (substring mode 0 5) "mode:")
-                            (setq mode (substring mode 6))))))
+                            (setq mode (substring mode 6)))))
              (intern mode))))))
 
 (defun ess-latex-find-chunk-index-buffer ()
