@@ -4,7 +4,7 @@
 ;; Maintainer: Paul Kinnucan
 ;; Keywords: java, tools
 
-;; Copyright (C) 1997, 1998, 2001, 2002, 2003 Paul Kinnucan.
+;; Copyright (C) 1997, 1998, 2001, 2002, 2003, 2004 Paul Kinnucan.
 
 ;; GNU Emacs is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -45,10 +45,6 @@
       (progn
 	(defvar w32-start-process-show-window nil)
 	(defvar w32-quote-process-args nil)
-	(defvar compilation-enter-directory-regexp-alist nil)
-	(defvar compilation-leave-directory-regexp-alist nil)
-	(defvar compilation-file-regexp-alist nil)
-	(defvar compilation-nomessage-regexp-alist nil)
 	(defvar last-nonmenu-event nil)
 	(defvar tags-table-format-hooks nil)
 	(defvar auto-insert nil)
@@ -74,6 +70,8 @@
       (defvar windowed-process-io nil)
       (unless (fboundp 'add-submenu)
 	(defun add-submenu (&rest args)))
+      (defun frame-property (&rest args))
+      (defun frame-highest-window (&rest args))
       (defun mswindows-cygwin-to-win32-path (&rest args))
       (defun ange-ftp-ftp-name (&rest args))
       (defun ange-ftp-get-file-entry (&rest args))
@@ -91,35 +89,45 @@
       (defun delete-extent (&rest args))
       (defun map-extents (&rest args))
       (defun extent-start-position (&rest args))))
-  (if jde-emacs21p
-      (defvar browse-url-new-window-p nil))
 
 
 ;; Define XEmacs- and Emacs-only variables 
 ;; and functions to avoid compiler warnings.
-  (if (not jde-xemacsp)
-      (defun make-event (&rest args)))
-  (if jde-emacs21p
-      (defun hscroll-window-column (&rest args)))
+(if (not jde-xemacsp)
+    (defun make-event (&rest args)))
+
+(if jde-emacs21p
+    (progn
+      (defvar browse-url-new-window-p nil)
+      (defun hscroll-window-column (&rest args))))
+
+;; Avoid undefined function error when compiling jde-help.
+(defun url-file-exists (&args))
+
+;; Required to compile jde-java-font-lock.el in versions
+;; of Emacs that lack the cc-fonts package.
+(eval-when-compile
+  (if (not (fboundp 'c-make-font-lock-search-function))
+      (defun c-make-font-lock-search-function (&rest args))))
 
 ;; From custom web page for compatibility between versions of custom:
-  (condition-case ()
-      (require 'custom)
-    (error nil))
-  (if (and (featurep 'custom) (fboundp 'custom-declare-variable))
-      nil ;; We've got what we needed
-    ;; We have the old custom-library, hack around it!
-    (defmacro defgroup (&rest args)
-      nil)
-    (defmacro defface (var values doc &rest args)
-      (` (progn
-	   (defvar (, var) (quote (, var)))
-	   ;; To make colors for your faces you need to set your .Xdefaults
-	   ;; or set them up ahead of time in your .emacs file.
-	   (make-face (, var))
-	   )))
-    (defmacro defcustom (var value doc &rest args)
-      (` (defvar (, var) (, value) (, doc)))))
+(condition-case ()
+    (require 'custom)
+  (error nil))
+(if (and (featurep 'custom) (fboundp 'custom-declare-variable))
+    nil	;; We've got what we needed
+  ;; We have the old custom-library, hack around it!
+  (defmacro defgroup (&rest args)
+    nil)
+  (defmacro defface (var values doc &rest args)
+    (` (progn
+	 (defvar (, var) (quote (, var)))
+	 ;; To make colors for your faces you need to set your .Xdefaults
+	 ;; or set them up ahead of time in your .emacs file.
+	 (make-face (, var))
+	 )))
+  (defmacro defcustom (var value doc &rest args)
+    (` (defvar (, var) (, value) (, doc)))))
 
 (provide 'jde-compat)
 
@@ -127,6 +135,17 @@
 
 ;;
 ;; $Log: jde-compat.el,v $
+;; Revision 1.6  2004/10/11 03:59:22  paulk
+;; Various updates.
+;;
+;; Revision 1.5  2004/07/01 14:04:39  jslopez
+;; Compatibility fix for emacs in CVS. Replaces jde-xemacsp check for boundp for
+;; the following variables: compilation-nomessage-regexp-alist,
+;; compilation-file-regexp-alist, compilation-leave-directory-regexp-alist,
+;; compilation-enter-directory-regexp-alist. Uses the compilation-mode without a
+;; parameter. The emacs in CVS does not contain the variables, or the parameter
+;; for compilation mode.
+;;
 ;; Revision 1.4  2003/10/23 03:57:11  paulk
 ;; Added compatibility fix for older versions of XEmacs. Thanks to Len Trigg.
 ;;
