@@ -31,7 +31,7 @@
 
 ;;; Commentary:
 
-;;      Plese insert `(require 'tm-vm)' in your ~/.vm file.
+;;      Please insert `(require 'tm-vm)' in your ~/.vm file.
 
 ;;; Code:
 
@@ -39,7 +39,6 @@
   (require 'tm-mail)
   (require 'vm)
   (require 'vm-window)
-  (require 'vm-easymenu)
   (require 'vm-macro))
 
 (require 'tm-edit)
