@@ -95,10 +95,9 @@ This value overrides `xwem-clgen-activate-new' if window is selected."
 
 ;;; Internal variables
 
-;;;###autoload
 (defvar xwem-clgen-map 
   (let ((map (make-sparse-keymap)))
-    (define-key map (kbd "H-c H-o") 'xwem-clgen-toggle-other-on-split)
+    (define-key map (xwem-kbd "H-c H-o") 'xwem-clgen-toggle-other-on-split)
     map)
   "Local keymap for generic clients.")
 
@@ -174,8 +173,6 @@ Probably we want to manage some client in newly created window."
   (xwem-clgen-activate-other (xwem-cl-selected) nwin))
 
 ;;; Othen-on-Split commands
-
-;;;###autoload(autoload 'xwem-clgen-turn-on-other-on-split "xwem-clgen" "" t)
 (define-xwem-command xwem-clgen-turn-on-other-on-split ()
   "Turn on `xwem-clgen-other-on-split' minor mode."
   (xwem-interactive)
@@ -185,7 +182,6 @@ Probably we want to manage some client in newly created window."
 
   (xwem-message 'info "Other on split minor mode is ON."))
 
-;;;###autoload(autoload 'xwem-clgen-turn-off-other-on-split "xwem-clgen" "" t)
 (define-xwem-command xwem-clgen-turn-off-other-on-split ()
   "Turn off `xwem-clgen-other-on-split' minor mode."
   (xwem-interactive)
@@ -195,7 +191,6 @@ Probably we want to manage some client in newly created window."
 
   (xwem-message 'info "Other on split minor mode is OFF."))
 
-;;;###autoload(autoload 'xwem-clgen-toggle-other-on-split "xwem-clgen" "" t)
 (define-xwem-command xwem-clgen-toggle-other-on-split (arg)
   "Toggle `xwem-clgen-other-on-split' minor mode.
 Negative ARG turns it off, positive turns it on."
@@ -226,7 +221,6 @@ Negative ARG turns it off, positive turns it on."
   :get 'xwem-cl-get-sys-prop
   :set 'xwem-cl-put-sys-prop)
 
-;;;###autoload
 (defun xwem-manage-generic (cl)
   "Manage method for generic clients."
 
