@@ -1,11 +1,11 @@
 ;;; xwem-keydefs.el --- Define standard XWEM keybindings.
 
-;; Copyright (C) 2003 by Free Software Foundation, Inc.
+;; Copyright (C) 2003-2005 by XWEM Org.
 
 ;; Author: Zajcev Evgeny <zevlg@yandex.ru>
 ;; Created: Fri Dec 12 15:47:42 MSK 2003
 ;; Keywords: xwem, xlib
-;; X-CVS: $Id: xwem-keydefs.el,v 1.11 2005/01/01 04:42:55 youngs Exp $
+;; X-CVS: $Id: xwem-keydefs.el,v 1.9 2004/11/30 22:02:48 lg Exp $
 
 ;; This file is part of XWEM.
 
@@ -81,7 +81,8 @@ Bindings:
 (xwem-keymap-put-prop 'xwem-help-prefix 'cursor 'xwem-cursor-help)
 (xwem-keymap-put-prop 'xwem-help-prefix 'prompt
   '(concat (single-key-description xwem-last-event) " "
-           (substitute-command-keys "(Type \\<xwem-help-map>\\[xwem-help-for-help] for help)")))
+           (substitute-command-keys
+            "(Type \\<xwem-help-map>\\[xwem-help-for-help] for help)")))
 (define-key xwem-global-map (xwem-kbd "H-h") 'xwem-help-prefix)
 
 ;;;###autoload(autoload 'xwem-root-prefix "xwem-keydefs" "Keymap for root window." nil 'keymap)
@@ -139,7 +140,8 @@ Bindings:
 (define-key xwem-global-map (xwem-kbd "H-a") 'xwem-launcher-prefix)
 
 (xwem-define-prefix-command 'xwem-launcher-other-win-prefix)
-(defvar xwem-launcher-other-win-map (symbol-function 'xwem-launcher-other-win-prefix)
+(defvar xwem-launcher-other-win-map
+  (symbol-function 'xwem-launcher-other-win-prefix)
   "Keymap for launcher in other window (\\<xwem-global-map>\\[xwem-launcher-other-win-prefix]) commands.
 
 Bindings:
@@ -207,6 +209,7 @@ Bindings:
 (define-key xwem-help-map (xwem-kbd "s") 'xwem-strokes-list)
 (define-key xwem-help-map (xwem-kbd "r") 'xwem-registers-list)
 (define-key xwem-help-map (xwem-kbd "H-y") 'xwem-help-cutbuffers)
+(define-key xwem-help-map (xwem-kbd "m") 'xwem-help-mode)
 
 ;; Universal argument
 (define-key xwem-global-map (xwem-kbd "H-u") 'xwem-universal-argument)
@@ -277,6 +280,7 @@ Bindings:
 
 ;; Clients commands
 (define-key xwem-global-map (xwem-kbd "H-x 7 z") 'xwem-client-iconify)
+(define-key xwem-global-map (xwem-kbd "H-x 7 Z") 'xwem-client-iconify-every)
 (define-key xwem-global-map (xwem-kbd "H-x 7 0") 'xwem-client-kill)
 (define-key xwem-global-map (xwem-kbd "H-x 7 1") 'xwem-client-iconify-others)
 (define-key xwem-global-map (xwem-kbd "H-x 7 2") 'xwem-client-run-copy)
@@ -293,6 +297,7 @@ Bindings:
 
 (define-key xwem-global-map (xwem-kbd "H-@") 'xwem-client-set-mark)
 (define-key xwem-global-map (xwem-kbd "H-SPC") 'xwem-client-set-mark)
+(define-key xwem-global-map (xwem-kbd "H-x H-x") 'xwem-client-exchange-selected-and-mark)
 (define-key xwem-global-map (xwem-kbd "H-x k") 'xwem-client-query-kill)
 
 (define-key xwem-global-map (xwem-kbd "H-x z") 'xwem-client-iconify)
@@ -354,12 +359,17 @@ Bindings:
 (define-key xwem-global-map (xwem-kbd "H-[") 'xwem-clswi-prev)
 (define-key xwem-global-map (xwem-kbd "H-}") 'xwem-clswi-next-other-window)
 (define-key xwem-global-map (xwem-kbd "H-{") 'xwem-clswi-prev-other-window)
+(define-key xwem-global-map (xwem-kbd "H-x 4 H-]") 'xwem-clswi-next-other-window)
+(define-key xwem-global-map (xwem-kbd "H-x 4 H-[") 'xwem-clswi-prev-other-window)
+(define-key xwem-global-map (xwem-kbd "H-o H-]") 'xwem-clswi-next-other-window)
+(define-key xwem-global-map (xwem-kbd "H-o H-[") 'xwem-clswi-prev-other-window)
 
 ;; Cutbuffers
 (define-key xwem-global-map (xwem-kbd "H-w") 'xwem-copy-cutbuffer)
 (define-key xwem-global-map (xwem-kbd "H-y") 'xwem-paste-cutbuffer)
 
 ;;; Misc bindings
+(define-key xwem-global-map (xwem-kbd "H-x H-f") 'xwem-open-file)
 ;; Making holer in frames
 (define-key xwem-global-map (xwem-kbd "H-x h") 'xwem-holer-prefix)
 ;; Log your work
