@@ -1,11 +1,11 @@
 ;;; xwem-minibuffer.el --- XWEM minibuffer support.
 
-;; Copyright (C) 2003 by Free Software Foundation, Inc.
+;; Copyright (C) 2003-2005 by XWEM Org.
 
 ;; Author: Zajcev Evgeny <zevlg@yandex.ru>
 ;; Created: Thu Dec  4 15:13:12 MSK 2003
 ;; Keywords: xwem, xlib
-;; X-CVS: $Id: xwem-minibuffer.el,v 1.11 2005/01/01 04:42:57 youngs Exp $
+;; X-CVS: $Id: xwem-minibuffer.el,v 1.9 2004/12/08 19:11:30 lg Exp $
 
 ;; This file is part of XWEM.
 
@@ -39,7 +39,7 @@
 (require 'xwem-manage)
 
 (eval-when-compile
-    (defvar x-emacs-application-class nil))
+  (defvar x-emacs-application-class nil))
 
 ;; Customization
 (defgroup xwem-minibuffer nil
@@ -55,11 +55,30 @@
 (defcustom xwem-minibuffer-bgcol "gray80"
   "*Background color to be used in `xwem-minib-frame'."
   :type 'color
+  :set (lambda (sym val)
+         (set sym val)
+         (when (and xwem-minibuffer
+                    (X-Win-p (xwem-minib-xwin xwem-minibuffer)))
+           (XSetWindowBackground
+            (xwem-dpy) (xwem-minib-xwin xwem-minibuffer)
+            (XAllocColor (xwem-dpy) (XDefaultColormap (xwem-dpy))
+                         (xwem-make-color xwem-minibuffer-bgcol)))
+           (XClearArea (xwem-dpy) (xwem-minib-xwin xwem-minibuffer) 0 0
+                       (X-Geom-width (xwem-minib-xgeom xwem-minibuffer))
+                       (X-Geom-height (xwem-minib-xgeom xwem-minibuffer)) nil)))
+  :initialize 'custom-initialize-default
   :group 'xwem-minibuffer)
 
 (defcustom xwem-minibuffer-font (face-font-name 'default)
   "*Font to be used in `xwem-minib-frame'.  May be nil or string."
   :type '(restricted-sexp :match-alternatives ('nil try-font-name))
+  :set (lambda (sym val)
+         (set sym val)
+         (when (and xwem-minibuffer
+                    (xwem-minib-frame xwem-minibuffer))
+           (set-face-font 'default xwem-minibuffer-font
+                          (xwem-minib-frame xwem-minibuffer))))
+  :initialize 'custom-initialize-default
   :group 'xwem-minibuffer)
 
 (defcustom xwem-minibuffer-height 1
@@ -93,14 +112,15 @@
   :group 'xwem-minibuffer)
 
 ;;;###xwem-autoload
-(defcustom xwem-minibuffer-border-width 3
+(defcustom xwem-minibuffer-border-width 2
   "Border width for `xwem-minibuffer'."
   :type 'number
   :set (lambda (sym val)
          (set sym val)
          (let ((cl (and xwem-minibuffer (xwem-minib-cl xwem-minibuffer))))
            (when (xwem-cl-p cl)
-             (xwem-client-set-property cl 'x-border-width xwem-minibuffer-border-width))))
+             (xwem-client-set-property
+              cl 'x-border-width xwem-minibuffer-border-width))))
   :initialize 'custom-initialize-default
   :group 'xwem-minibuffer)
 
@@ -128,11 +148,13 @@
          (let ((cl (and xwem-minibuffer (xwem-minib-cl xwem-minibuffer))))
            (when (xwem-cl-p cl)
              (xwem-set-face-foreground
-              'x-border-face xwem-minibuffer-active-border-color '(selected) cl)
+              'x-border-face xwem-minibuffer-active-border-color
+              '(selected) cl)
              (xwem-client-set-property
               cl 'x-border-color
               (xwem-face-foreground
-               'x-border-face (and (xwem-cl-selected-p cl) '(selected)) cl)))))
+               'x-border-face (and (xwem-cl-selected-p cl)
+                                   '(selected)) cl)))))
   :initialize 'custom-initialize-default
   :group 'xwem-minibuffer)
 
@@ -149,8 +171,7 @@ when it focused."
   :group 'xwem-minibuffer)
 
 (defcustom xwem-minibuffer-autohide-timeout nil
-  "*Non-nil number, mean xwem minibuffer will be autohided, after that many seconds.
-NOT YET IMPLEMENTED."
+  "*Non-nil mean xwem minibuffer will be autohided, after that many seconds."
   :type '(choice (const :tag "Disabled" nil)
                  (number :tag "Seconds"))
   :set (lambda (sym value)
@@ -164,6 +185,20 @@ NOT YET IMPLEMENTED."
   :initialize 'custom-initialize-default
   :group 'xwem-minibuffer)
 
+(defcustom xwem-minibiffer-always-on-top-rank 20
+  "*Always on top rank or nil."
+  :type '(choice (const :tag "No rank" nil)
+                 (number :tag "Rank"))
+  :set (lambda (sym value)
+         (set sym value)
+         (let ((xwin (xwem-minib-xwin xwem-minibiffer)))
+           (when xwin
+             (if value
+                 (xwem-misc-set-xwin-always-on-top xwin value)
+               (xwem-misc-unset-always-on-top xwin)))))
+  :initialize 'custom-initialize-default
+  :group 'xwem-minibuffer)
+
 (defcustom xwem-minibuffer-raise-when-active t
   "*Non-nil mean xwem minibuffer is raised when activated."
   :type 'boolean
@@ -175,7 +210,8 @@ NOT YET IMPLEMENTED."
   :group 'xwem-minibuffer)
 
 (defcustom xwem-minibuffer-set-default-minibuffer-frame t
-  "*Non-nil mean that xwem minibuffer frame will be set as `default-minibuffer-frame'."
+  "*Non-nil mean that xwem minibuffer frame will be set as `default-minibuffer-frame'.
+Modify this variable only if you know what you are doing."
   :type 'boolean
   :group 'xwem-minibuffer)
 
@@ -213,6 +249,12 @@ never get smaller while it is active."
 
 (defcustom xwem-minib-specifiers
   '((default-toolbar-visible-p . nil)
+
+    ;; Gutters
+    (default-gutter-visible-p . t)
+    (top-gutter . nil)
+    (top-gutter-border-width . 1)
+
     (menubar-visible-p . nil)
     (horizontal-scrollbar-visible-p . nil)
     ((face-font 'default) . xwem-minibuffer-font))
@@ -236,28 +278,36 @@ never get smaller while it is active."
 (defvar xwem-minibuffer-map 
   (let ((map (make-sparse-keymap)))
     (define-key map (xwem-kbd "H-g") 'minibuffer-keyboard-quit)
+    (define-key map (xwem-kbd "C-<button1>") 'xwem-client-imove)
     map)
   "Keymap used while in xwem.")
 
 
 (defun xwem-minib-apply-specifiers (frame)
   "Apply `xwem-minib-specifiers' to FRAME."
-  (mapc (lambda (spc)
-          (set-specifier (eval (car spc)) (eval (cdr spc)) frame nil 'remove-locale))
+  (mapc #'(lambda (spc)
+            (set-specifier (eval (car spc)) (eval (cdr spc))
+                           frame nil 'remove-locale))
         xwem-minib-specifiers))
 
 (defun xwem-minib-create ()
   "Create minibuffer that will be used by xwem, or use existen."
-  (let ((mframe (xwem-misc-find-frame xwem-minibuffer-name)))
-    (when (null mframe)
-      ;; xwem minib not yet created
-      (setq mframe (make-frame minibuffer-frame-plist (default-x-device))))
+  (let ((mframe (or (xwem-misc-find-frame xwem-minibuffer-name
+                                          (minibuffer-frame-list))
+                    (make-frame minibuffer-frame-plist
+                                (default-x-device))))
+        (dd default-directory))
 
     (setf (xwem-minib-frame xwem-minibuffer) mframe)
-
     ;; Set specifiers values for MFRAME
     (xwem-minib-apply-specifiers mframe)
-    (redraw-frame mframe t)       ; KEEP THIS!
+    (redraw-frame mframe t)             ; KEEP THIS!
+
+    ;; Hack over default-directory for minibuffer buffer
+    (with-current-buffer
+        (window-buffer (frame-root-window mframe))
+      (setq default-directory dd))
+
     mframe))
 
 (defmacro xwem-cl-minibuffer (cl)
@@ -288,11 +338,14 @@ never get smaller while it is active."
 (defun xwem-manage-minibuffer (cl)
   "Manage method for xwem minibuffers."
   (let* ((xgeom (make-X-Geom :x 0       ; XXX
-                             :y (X-Geom-height (xwem-rootgeom))
+                             :y (- (X-Geom-height (xwem-rootgeom))
+                                   xwem-minibuffer-outer-border-width
+                                   xwem-minibuffer-outer-border-width)
                              :width (- (X-Geom-width (xwem-rootgeom))
                                        xwem-minibuffer-outer-border-width
                                        xwem-minibuffer-outer-border-width)
-                             :height (X-Geom-height-with-borders (xwem-cl-xgeom cl))
+                             :height (X-Geom-height-with-borders
+                                      (xwem-cl-xgeom cl))
                              :border-width xwem-minibuffer-outer-border-width))
          (minib (make-xwem-minib
                  :frame (xwem-misc-find-emacs-frame cl)
@@ -315,8 +368,9 @@ never get smaller while it is active."
     ;; Setup window a little
     (when xwem-minibuffer-outer-border-color
       (XSetWindowBorder (xwem-dpy) (xwem-minib-xwin minib)
-                        (XAllocNamedColor (xwem-dpy) (XDefaultColormap (xwem-dpy))
-                                          xwem-minibuffer-outer-border-color)))
+                        (XAllocNamedColor
+                         (xwem-dpy) (XDefaultColormap (xwem-dpy))
+                         xwem-minibuffer-outer-border-color)))
                                           
     ;; Save CL's minibuffer
     (setf (xwem-cl-minibuffer cl) minib)
@@ -329,7 +383,8 @@ never get smaller while it is active."
     (xwem-client-set-property cl 'x-border-width xwem-minibuffer-border-width)
     (xwem-client-set-property cl 'x-border-color
                               (xwem-face-foreground 'x-border-face
-                                (and (xwem-cl-selected-p cl) '(selected)) cl))
+                                                    (and (xwem-cl-selected-p cl)
+                                                         '(selected)) cl))
 
     ;; Reparent xwem minib client to parent
     ;; XXX XXX
@@ -348,6 +403,11 @@ never get smaller while it is active."
     (xwem-refit cl)
     (XMapWindow (xwem-dpy) (xwem-minib-xwin minib))
 
+    ;; Set always on top rank (if any)
+    (when xwem-minibiffer-always-on-top-rank
+      (xwem-misc-set-xwin-always-on-top
+       (xwem-minib-xwin minib) xwem-minibiffer-always-on-top-rank))
+
     ;; Set default minibuffer, if not already set
     (unless (xwem-cl-p (xwem-minib-cl xwem-minibuffer))
       (setq xwem-minibuffer minib)
@@ -355,7 +415,8 @@ never get smaller while it is active."
         (setq default-minibuffer-frame (xwem-minib-frame xwem-minibuffer))))
 
     ;; Now activate minibuffer
-    (xwem-activate cl)))
+    (xwem-activate cl)
+    (xwem-minib-apply-state-1 minib )))
 
 (define-xwem-deffered xwem-minib-apply-pxgeom (minib)
   "Apply MINIB's parent geometry to life."
@@ -375,28 +436,40 @@ never get smaller while it is active."
 
 (defun xwem-refit-minibuffer (cl)
   "Refit xwem minibuffer client CL."
-  (xwem-debug 'xwem-minib "Refiting ..")
+  (xwem-debug 'xwem-misc "Minib: Refiting .. to %S" '(xwem-cl-new-xgeom cl))
 
   (let ((cl-xgeom (xwem-cl-xgeom cl))
+        (cl-nx (and (xwem-cl-new-xgeom cl)
+                    (X-Geom-x (xwem-cl-new-xgeom cl))))
         (pxgeom (xwem-minib-xgeom (xwem-cl-minibuffer cl))))
-
     ;; Adjust geometry a little to fill into xwem-minib-xwin and apply
     ;; changes to life
     (xwem-cl-apply-new-xgeom cl)
-    (xwem-cl-correct-size-for-size cl
-      (make-X-Geom :x 0 :y 0
-                   :width (X-Geom-width-with-borders cl-xgeom)
-                   :height (X-Geom-height-with-borders cl-xgeom)
-                   :border-width (X-Geom-border-width cl-xgeom))
-      'left 'top)
+    (when cl-nx
+      ;; CL has new x location - handle it
+      (setf (X-Geom-x pxgeom) (X-Geom-x cl-xgeom))
+      (setf (X-Geom-width pxgeom)
+            (- (X-Geom-width (xwem-rootgeom))
+               (X-Geom-x pxgeom)
+               xwem-minibuffer-outer-border-width
+               xwem-minibuffer-outer-border-width))
+      (xwem-minib-apply-pxgeom (xwem-cl-minibuffer cl)))
+    (xwem-cl-correct-size-for-size
+     cl
+     (make-X-Geom :x 0 :y 0
+                  :width (X-Geom-width-with-borders cl-xgeom)
+                  :height (X-Geom-height-with-borders cl-xgeom)
+                  :border-width (X-Geom-border-width cl-xgeom))
+     'left 'top)
     (xwem-cl-apply-xgeom cl)
 
     ;; Check maybe parent need to be resized/moved?
     (unless (= (X-Geom-height-with-borders cl-xgeom)
                (X-Geom-height pxgeom))
-      (decf (X-Geom-y pxgeom)
-            (- (X-Geom-height-with-borders cl-xgeom)
-               (X-Geom-height pxgeom)))
+      (when (eq (xwem-cl-state cl) 'active)
+        (decf (X-Geom-y pxgeom)
+              (- (X-Geom-height-with-borders cl-xgeom)
+                 (X-Geom-height pxgeom))))
       (setf (X-Geom-height pxgeom)
             (X-Geom-height-with-borders cl-xgeom))
       (xwem-minib-apply-pxgeom (xwem-cl-minibuffer cl)))))
@@ -417,7 +490,8 @@ never get smaller while it is active."
   (when (numberp xwem-minibuffer-autohide-timeout)
     (xwem-minibuffer-disable-autohide-timer cl)
     (xwem-cl-put-sys-prop cl 'auto-hide-timer
-      (add-timeout xwem-minibuffer-autohide-timeout 'xwem-minibuffer-autohide-timer cl))))
+      (add-timeout xwem-minibuffer-autohide-timeout
+                   'xwem-minibuffer-autohide-timer cl))))
   
 (define-xwem-deffered xwem-minib-apply-state (minib)
   "Apply xwem minibuffer MINIB's state to life."
@@ -438,14 +512,15 @@ never get smaller while it is active."
         (XMoveWindow (xwem-dpy) (xwem-minib-xwin minib)
                      (X-Geom-x (xwem-minib-xgeom minib))
                      (X-Geom-y (xwem-minib-xgeom minib)))
+        (XFlush (xwem-dpy))
         (when param
           (sit-for param t))
         (setq i (1+ i)))
       (xwem-minib-put-prop minib 'saved-state state)
 
       (when (and xwem-minibuffer-raise-when-active
-		 (eq state 'active))
-	(XRaiseWindow (xwem-dpy) (xwem-minib-xwin minib))))
+                 (eq state 'active))
+        (xwem-misc-raise-xwin (xwem-minib-xwin minib))))
 
     (when (eq state 'active)
       (xwem-minibuffer-enable-autohide-timer (xwem-minib-cl minib)))))
@@ -481,18 +556,18 @@ never get smaller while it is active."
 ;; Events handler
 (define-xwem-deffered xwem-minib-focusin-autoraise (minib)
   "Mainly for use in `xwem-minibuffer-focusout-hook'."
-  (XRaiseWindow (xwem-dpy) (xwem-minib-xwin minib)))
+  (xwem-misc-raise-xwin (xwem-minib-xwin minib)))
 
 (define-xwem-deffered xwem-minib-focusout-autolower (minib)
   "Mainly for use in `xwem-minibuffer-focusout-hook'."
-  (XLowerWindow (xwem-dpy) (xwem-minib-xwin minib)))
+  (xwem-misc-lower-xwin (xwem-minib-xwin minib)))
 
 ;;;###autoload(autoload 'xwem-minibuffer-activate "xwem-minibuffer" "" t)
 (define-xwem-command xwem-minibuffer-activate ()
   "Switch to xwem minibuffer if it is in active state."
   (xwem-interactive)
   (if (eq (active-minibuffer-window)
-	  (frame-selected-window (xwem-minib-frame xwem-minibuffer)))
+          (frame-selected-window (xwem-minib-frame xwem-minibuffer)))
       (xwem-select-client (xwem-minib-cl xwem-minibuffer))
     (xwem-activate (xwem-minib-cl xwem-minibuffer))))
 
@@ -516,16 +591,19 @@ never get smaller while it is active."
     (setq initial-frame-plist
           (plist-put initial-frame-plist 'minibuffer nil)))
   (setq initial-frame-plist
-	(plist-put initial-frame-plist 'wait-for-wm nil))
+        (plist-put initial-frame-plist 'wait-for-wm nil))
 
   ;; Adjust default frame params
   (unless xwem-minibuffer-emacs-frames-has-minibuffer
     (setq default-x-frame-plist
-          (plist-put default-frame-plist 'minibuffer nil)))
+          (plist-put default-x-frame-plist 'minibuffer nil)))
   (setq default-x-frame-plist
-	(plist-put default-frame-plist 'wait-for-wm nil))
+        (plist-put default-x-frame-plist 'wait-for-wm nil))
 
+  ;; Create XEmacs minibuffer only frame for xwem minibuffer
   (setq xwem-minibuffer (make-xwem-minib))
+  (setf (xwem-minib-frame xwem-minibuffer)
+        (make-initial-minibuffer-frame nil))
   (xwem-message 'init "Initializing minibuffer ... done"))
 
 ;;; Resize-minibuffer mode
@@ -545,14 +623,14 @@ respectively."
     (save-excursion
       (save-restriction
         (widen)
-	(narrow-to-region start end)
-	(goto-char start)
+        (narrow-to-region start end)
+        (goto-char start)
         (vertical-motion (buffer-size))))))
 
 (defun xwem-minib-rsz-restore ()
   "Restore xwem minibuffer size."
   (xwem-minib-rsz-resize xwem-minib-rsz-saved-height)
-  (XLowerWindow (xwem-dpy) (xwem-minib-xwin xwem-minibuffer)))
+  (xwem-misc-lower-xwin (xwem-minib-xwin xwem-minibuffer)))
 
 (defun xwem-minib-rsz-check ()
   "Called in `pre-command-hook'.
@@ -573,13 +651,14 @@ If NEW-HEIGHT is ommited, current window height considered."
            (set-frame-size frame (frame-width frame) lines)
            (setq n (frame-pixel-height frame))
 
-           (XRaiseWindow (xwem-dpy) (xwem-minib-xwin xwem-minibuffer))
+           (xwem-misc-raise-xwin (xwem-minib-xwin xwem-minibuffer))
 
            ;; Adjust xwem minibuffer cl height
            (xwem-client-resize (xwem-minib-cl xwem-minibuffer) nil
-                               (+ (X-Geom-height (xwem-minib-cl-xgeom xwem-minibuffer))
+                               (+ (X-Geom-height
+                                   (xwem-minib-cl-xgeom xwem-minibuffer))
                                   (- n w)))
-          ))))
+           ))))
 
 (defun xwem-minib-rsz-setup ()
   "Setup xwem resize minibuffer mode for xwem minibuffer."
@@ -609,6 +688,112 @@ With prefix ARG start, without - stop."
       (add-hook 'minibuffer-setup-hook 'xwem-minib-rsz-setup)
     (remove-hook 'minibuffer-setup-hook 'xwem-minib-rsz-setup)))
 
+;;; xwem minibuffer modeline using top gutter
+(defvar xwem-modeline-format
+  '("--"
+    (symbol-name (xwem-cl-manage-type cl))
+    ": "
+    "["
+    (or (car (xwem-client-application cl))
+        "unknown")
+    "]"
+
+    ;; Some additional info
+    " "
+    (if (xwem-cl-marked-p cl) "*" "-")
+    (let ((reg (xwem-client-property cl 'register)))
+      (if reg (list (format "%c" reg) 'bold) "-"))
+    " "
+
+    (list (xwem-client-name cl) 'modeline-buffer-id)
+    "   "
+    ;; Minor modes
+    "("
+    (mapconcat 'identity
+               (delq nil (mapcar #'(lambda (mm)
+                                     (and (symbol-value (car mm))
+                                          (cadr mm)))
+                                 xwem-minor-mode-alist)) " ")
+    ")"
+    "----"
+    (let ((usz (xwem-cl-get-usize cl)))
+      (format "%dx%d" (car usz) (cdr usz)))
+    "--")
+  "Modeline format.")
+(xwem-make-variable-client-local 'xwem-modeline-format)
+
+(defun xwem-modeline-regenerate ()
+  "Regenerate modeline string."
+  (mapconcat #'(lambda (me)
+                 (let ((cl (xwem-cl-selected)))
+                   (setq cl cl)         ; shutup compiler
+                   (condition-case nil
+                       (let ((str "") (faces nil))
+                         (setq me (eval me))
+                         (if (listp me)
+                             (setq str (copy-sequence (car me))
+                                   faces (cdr me))
+                           (setq str (copy-sequence me)
+                                 faces nil))
+                         (xwem-str-with-faces str (append '(modeline) faces)))
+                     (t "<error>"))))
+             xwem-modeline-format ""))
+
+(define-xwem-deffered xwem-modeline-redraw (&optional cl)
+  "Redraw xwem modeline."
+  (if (xwem-cl-alive-p cl)
+      (when (xwem-cl-selected-p cl)
+        (xwem-modeline-redraw))
+
+    (let* ((str (xwem-modeline-regenerate))
+           (mw (frame-width (xwem-minib-frame xwem-minibuffer))))
+      (set-specifier top-gutter (substring str 0 (and (> (length str) mw) mw))
+                     (xwem-minib-frame xwem-minibuffer))
+      )))
+
+;;;###autoload(autoload 'xwem-modeline-enable "xwem-minibuffer" nil t)
+(define-xwem-command xwem-modeline-enable ()
+  "Enable modeline."
+  (xwem-interactive)
+
+  (add-hook 'xwem-cl-change-hook 'xwem-modeline-redraw)
+  (add-hook 'xwem-client-select-hook 'xwem-modeline-redraw)
+
+  (set-specifier top-gutter-visible-p t
+                 (xwem-minib-frame xwem-minibuffer))
+  ;; Start showing gutter
+  (xwem-modeline-redraw-1)
+  ;; Fix xwem minibuffer height size
+  (xwem-client-resize
+   (xwem-minib-cl xwem-minibuffer) nil
+   (+ (X-Geom-height-with-borders (xwem-minib-cl-xgeom xwem-minibuffer))
+      (gutter-pixel-height 'top (xwem-minib-frame xwem-minibuffer))
+      (specifier-instance top-gutter-border-width
+                          (xwem-minib-frame xwem-minibuffer))
+      (specifier-instance top-gutter-border-width
+                          (xwem-minib-frame xwem-minibuffer)))))
+
+;;;###autoload(autoload 'xwem-modeline-disable "xwem-minibuffer" nil t)
+(define-xwem-command xwem-modeline-disable ()
+  "Disable modeline."
+  (xwem-interactive)
+
+  (remove-hook 'xwem-cl-change-hook 'xwem-modeline-redraw)
+  (remove-hook 'xwem-client-select-hook 'xwem-modeline-redraw)
+
+  ;; Fix xwem minibuffer height size
+  (xwem-client-resize
+   (xwem-minib-cl xwem-minibuffer) nil
+   (- (X-Geom-height-with-borders (xwem-minib-cl-xgeom xwem-minibuffer))
+      (gutter-pixel-height 'top (xwem-minib-frame xwem-minibuffer))
+      (specifier-instance top-gutter-border-width
+                          (xwem-minib-frame xwem-minibuffer))
+      (specifier-instance top-gutter-border-width
+                          (xwem-minib-frame xwem-minibuffer))))
+  ;; Stop showing gutter
+  (set-specifier top-gutter nil
+                 (xwem-minib-frame xwem-minibuffer)))
+
 
 (provide 'xwem-minibuffer)
 
@@ -616,15 +801,15 @@ With prefix ARG start, without - stop."
 ;; Define application
 (add-to-list 'xwem-applications-alist
              `("xemacs-xwem-minibuffer"
-                   (and (class-name ,(concat "^" x-emacs-application-class "$"))
-                        (class-inst ,(concat "^" xwem-minibuffer-name "$")))))
+               (and (class-name ,(concat "^" x-emacs-application-class "$"))
+                    (class-inst ,(concat "^" xwem-minibuffer-name "$")))))
 
 ;; Add manage type
 (define-xwem-manage-model minibuffer
   "Managing model for xwem minibuffer."
   :cl-properties '(dummy-client-p t
-                   skip-deselect t
-                   override-skip-deselect t)
+                                  skip-deselect t
+                                  override-skip-deselect t)
   :match-spec '(application "xemacs-xwem-minibuffer")
 
   :manage-method 'xwem-manage-minibuffer
