@@ -1,12 +1,12 @@
 ;;; xwem-netwm.el --- NETWM stuff.
 
-;; Copyright (C) 2004 by Free Software Foundation, Inc.
+;; Copyright (C) 2004,2005 by XWEM Org.
 
 ;; Author: Zajcev Evgeny <zevlg@yandex.ru>
 ;;         Steve Youngs  <steve@youngs.au.com>
 ;; Created: Sat May 15 19:44:58 MSD 2004
 ;; Keywords: xwem
-;; X-CVS: $Id: xwem-netwm.el,v 1.5 2005/01/01 04:42:58 youngs Exp $
+;; X-CVS: $Id: xwem-netwm.el,v 1.3 2004/12/05 22:37:35 lg Exp $
 
 ;; This file is part of XWEM.
 
@@ -51,6 +51,18 @@
   :type 'boolean
   :group 'xwem-fullscreen)
 
+(defcustom xwem-fullscreen-iconify-when-inactive t
+  "*Non-nil mean iconify fullscreen clients, when they gets inactive/deselected."
+  :type 'boolean
+  :group 'xwem-fullscreen)
+
+(defcustom xwem-fullscreen-ai-rank '(1000 . 0)
+  "*Always on top rank pair for active and inactive states."
+  :type '(cons :tag "Rank pair"
+               (number :tag "Active rank")
+               (number :tag "Inactive rank"))
+  :group 'xwem-fullscreen)
+
 ;;; Internal variables
 
 ;;; Constants
@@ -152,8 +164,8 @@
   "Initialize netwm stuff."
   (xwem-message 'init "Initializing NET_WM support ...")
 
-  (mapc (lambda (name)
-          (XInternAtom (xwem-dpy) name))
+  (mapc #'(lambda (name)
+            (XInternAtom (xwem-dpy) name))
         xwem-nwm-atom-names)
 
   ;; Add client message handler on root window
@@ -200,18 +212,21 @@
 
 (defun xwem-nwm-root-clnmsg (xdpy xwin xev)
   "Dispatch ClientMessage event on root window."
-  (cond ((string= (X-Atom-name (X-Event-xclient-atom xev)) _NET_CURRENT_DESKTOP)
+  (cond ((string= (X-Atom-name (X-Event-xclient-atom xev))
+                  _NET_CURRENT_DESKTOP)
          (let ((num (truncate (caar (X-Event-xclient-msg xev)))))
            (xwem-frame-switch-nth num)))
 
         ((string= (X-Atom-name (X-Event-xclient-atom xev)) _NET_WM_STATE)
-         (xwem-debug 'xwem-misc "_NET_WM_STATE: -> %S" '(X-Event-xclient-msg xev))
+         (xwem-debug 'xwem-misc "_NET_WM_STATE: -> %S"
+                     '(X-Event-xclient-msg xev))
          
          (let ((vop (caar (X-Event-xclient-msg xev)))
                (stype (caadr (X-Event-xclient-msg xev)))
                (cl (xwem-xwin-cl xwin)))
            ;; XXX Can handle only FULLSCREEN state
-           (when (= stype (X-Atom-id (XInternAtom (xwem-dpy) _NET_WM_STATE_FULLSCREEN)))
+           (when (= stype (X-Atom-id (XInternAtom
+                                      (xwem-dpy) _NET_WM_STATE_FULLSCREEN)))
              (cond ((= vop 0)
                     (xwem-toggle-fullscreen cl 'off))
                    ((= vop 1)
@@ -254,8 +269,8 @@ If STATE is nil, then remove property."
   (XChangeProperty (xwem-dpy) (xwem-rootwin)
                    (XInternAtom (xwem-dpy) _NET_SUPPORTED)
                    XA-atom X-format-32 X-PropModeReplace
-                   (mapcar (lambda (name)
-                             (XInternAtom (xwem-dpy) name))
+                   (mapcar #'(lambda (name)
+                               (XInternAtom (xwem-dpy) name))
                            xwem-nwm-supported)))
 
 (defun xwem-nwm-set-number-of-desktops (&rest notused)
@@ -263,7 +278,7 @@ If STATE is nil, then remove property."
   (XChangeProperty (xwem-dpy) (xwem-rootwin)
                    (XInternAtom (xwem-dpy) _NET_NUMBER_OF_DESKTOPS)
                    XA-cardinal X-format-32 X-PropModeReplace
-                   (list (length (xwem-frames-list)))))
+                   (list (length (xwem-frames-list 'desktop)))))
 
 (defun xwem-nwm-set-desk-geometry (&optional frame &rest notused)
   "Set _NET_DESKTOP_GEOMETRY."
@@ -273,7 +288,8 @@ If STATE is nil, then remove property."
     (XChangeProperty (xwem-dpy) (xwem-rootwin)
                      (XInternAtom (xwem-dpy) _NET_DESKTOP_GEOMETRY)
                      XA-cardinal X-format-32 X-PropModeReplace
-                     (list (xwem-frame-width frame) (xwem-frame-height frame)))))
+                     (list (xwem-frame-width frame)
+                           (xwem-frame-height frame)))))
 
 (defun xwem-nwm-set-current-desk (&optional frame &rest notused)
   "Set _NET_CURRENT_DESKTOP."
@@ -302,7 +318,8 @@ If STATE is nil, then remove property."
   (XChangeProperty (xwem-dpy) (xwem-rootwin)
                    (XInternAtom (xwem-dpy) _NET_ACTIVE_WINDOW)
                    XA-window X-format-32 X-PropModeReplace
-                   (list (and (and (xwem-cl-p (xwem-cl-selected)) (X-Win-id (xwem-cl-xwin (xwem-cl-selected))))))))
+                   (list (and (xwem-cl-p (xwem-cl-selected))
+                              (X-Win-id (xwem-cl-xwin (xwem-cl-selected)))))))
 
 (defun xwem-nwm-set-wm-window-type (thing)
   "Set _NET_WM_WINDOW_TYPE."
@@ -431,11 +448,30 @@ It is an alias for `xwem-toggle-fullscreen' command."
 
   (xwem-toggle-fullscreen cl))
 
-;;;###autoload(progn (autoload 'xwem:-fullscreen-manage "xwem-netwm") (put 'managa 'fullscreen 'xwem:-fullscreen-manage))
+;;;###autoload
 (defun xwem-manage-fullscreen (cl)
   "Manage method for fullscreen client CL."
-  (XReparentWindow (xwem-dpy) (xwem-cl-xwin cl)
-                   (xwem-rootwin) 0 0)
+  ;; Find out the place where to manage client
+  (let ((tpnt (car (XTranslateCoordinates
+                    (xwem-dpy) (xwem-cl-xwin cl)
+                    (xwem-rootwin) 0 0)))
+        (xin (X-XIneramaQueryScreens (xwem-dpy)))
+        (rx 0) (ry 0))
+    ;; Xinerama stuff
+    (when (car xin)
+      ;; XInerama enabled
+      (while (setq xin (cdr xin))
+        (when (and (>= (X-Point-x tpnt) (X-Rect-x (car xin)))
+                   (<= (X-Point-x tpnt)
+                       (+ (X-Rect-x (car xin)) (X-Rect-width (car xin))))
+                   (>= (X-Point-y tpnt) (X-Rect-y (car xin)))
+                   (<= (X-Point-y tpnt)
+                       (+ (X-Rect-y (car xin)) (X-Rect-height (car xin)))))
+          (setq rx (X-Rect-x (car xin))
+                ry (X-Rect-y (car xin))
+                xin nil))))
+
+    (XReparentWindow (xwem-dpy) (xwem-cl-xwin cl) (xwem-rootwin) rx ry))
 
   ;; Set geometry to initial
   (setf (xwem-cl-new-xgeom cl)
@@ -446,8 +482,9 @@ It is an alias for `xwem-toggle-fullscreen' command."
   (xwem-use-local-map xwem-fullscreen-mode-map cl)
   (xwem-select-client cl)               ; XXX
   
-  (run-hook-with-args 'xwem-fullscreen-mode-hook cl)
-  )
+  (run-hook-with-args 'xwem-fullscreen-mode-hook cl))
+
+;;;###autoload(put 'manage 'fullscreen 'xwem-manage-fullscreen)
 
 (defun xwem-fullscreen-refit-full (cl)
   "Refit CL to fullscreen."
@@ -494,13 +531,26 @@ It is an alias for `xwem-toggle-fullscreen' command."
 
 (define-xwem-deffered xwem-fullscreen-apply-state (cl)
   "Apply CL's state to life."
-  (cond ((eq (xwem-cl-state cl) 'active)
-         (XMapWindow (xwem-dpy) (xwem-cl-xwin cl))
-         (XRaiseWindow (xwem-dpy) (xwem-cl-xwin cl)))
+  (when (xwem-cl-p cl)
+    (case (xwem-cl-state cl)
+      (active
+       (xwem-misc-set-xwin-always-on-top
+        (xwem-cl-xwin cl) (car xwem-fullscreen-ai-rank))
+       (XMapWindow (xwem-dpy) (xwem-cl-xwin cl)))
         
-        ((memq (xwem-cl-state cl) '(inactive iconified))
-         (XLowerWindow (xwem-dpy) (xwem-cl-xwin cl))
-         (XUnmapWindow (xwem-dpy) (xwem-cl-xwin cl)))))
+      (inactive
+       (xwem-misc-set-xwin-always-on-top
+        (xwem-cl-xwin cl) (cdr xwem-fullscreen-ai-rank))
+       (when (and xwem-fullscreen-iconify-when-inactive
+                  ;; XXX skip dummy clients and special
+                  (not (or (xwem-dummy-client-p (xwem-cl-selected))
+                           (xwem-special-p (xwem-cl-selected)))))
+         (xwem-iconify cl)))
+
+      (iconified
+       (xwem-misc-set-xwin-always-on-top
+        (xwem-cl-xwin cl) (cdr xwem-fullscreen-ai-rank))
+       (XUnmapWindow (xwem-dpy) (xwem-cl-xwin cl))))))
          
 (defun xwem-activate-fullscreen (cl &optional type)
   "Activate method for fullscreen client CL."
@@ -509,6 +559,11 @@ It is an alias for `xwem-toggle-fullscreen' command."
 (defun xwem-deactivate-fullscreen (cl &optional type)
   "Deactivate fullscreen client CL."
   (cond ((eq type 'deactivate)
+         (xwem-fullscreen-apply-state cl))
+
+        ((eq type 'deselect)
+         (when (xwem-cl-active-p cl)
+           (xwem-client-change-state cl 'inactive))
          (xwem-fullscreen-apply-state cl))))
 
 (defun xwem-iconify-fullscreen (cl)
@@ -518,8 +573,9 @@ It is an alias for `xwem-toggle-fullscreen' command."
 ;;; Additional methods
 (define-xwem-method other-client fullscreen (cl)
   "Return fullscreen client other then CL."
-  (or (xwem-cl-other cl (xwem-clients-list 'xwem-cl-fullscreen-p) t)
-      (and xwem-fullscreen-switch-any-other(xwem-cl-other cl nil t))))
+  (or (xwem-cl-other cl :clients (xwem-clients-list 'xwem-cl-fullscreen-p))
+      (and xwem-fullscreen-switch-any-other
+	   (xwem-cl-other cl))))
 
 ;;;; ---- Fullscreen Commands ----
 
@@ -568,7 +624,8 @@ Optional argument FORCE is one of:
 (define-xwem-manage-model fullscreen
   "Managing model to show client at fullscreen size."
   :match-spec '(function xwem-netwm-fullscreen-p)
-  
+  :manage-properties '(omit-aspect-ratio t) ; disregard aspect ratio
+
   :manage-method 'xwem-manage-fullscreen
   :activate-method 'xwem-activate-fullscreen
   :deactivate-method 'xwem-deactivate-fullscreen
