@@ -1,11 +1,11 @@
 ;;; xwem-tabbing.el --- Tabs in XWEM frames.
 
-;; Copyright (C) 2003 by Free Software Foundation, Inc.
+;; Copyright (C) 2003-2005 by XWEM Org.
 
 ;; Author: Zajcev Evgeny <zevlg@yandex.ru>
 ;; Created: Sun Dec  7 18:35:15 MSK 2003
 ;; Keywords: xwem, xlib
-;; X-CVS: $Id: xwem-tabbing.el,v 1.7 2005/01/01 04:43:01 youngs Exp $
+;; X-CVS: $Id: xwem-tabbing.el,v 1.5 2004/12/05 05:52:31 youngs Exp $
 
 ;; This file is part of XWEM.
 
@@ -133,6 +133,55 @@
   :group 'xwem-tab
   :group 'xwem-faces)
 
+;; Another interface to customize tabber fonts
+(defcustom xwem-tabber-font:frame.selected-tab.selected nil
+  "Font to be used in selected tab of selected frame."
+  :type '(restricted-sexp :match-alternatives (nil try-font-name))
+  :set (lambda (sym val)
+         (set sym val)
+         (xwem-set-face-font
+          'xwem-tabber-face
+          (or val (xwem-face-font 'xwem-tabber-face '(default)))
+          '(frame-selected tab-selected)))
+  :initialize 'custom-initialize-default
+  :group 'xwem-tab)
+
+(defcustom xwem-tabber-font:frame.selected-tab.nonselected nil
+  "Font to be used in selected tab of selected frame."
+  :type '(restricted-sexp :match-alternatives (nil try-font-name))
+  :set (lambda (sym val)
+         (set sym val)
+         (xwem-set-face-font
+          'xwem-tabber-face
+          (or val (xwem-face-font 'xwem-tabber-face '(default)))
+          '(frame-selected tab-nonselected)))
+  :initialize 'custom-initialize-default
+  :group 'xwem-tab)
+
+(defcustom xwem-tabber-font:frame.nonselected-tab.selected nil
+  "Font to be used in selected tab of selected frame."
+  :type '(restricted-sexp :match-alternatives (nil try-font-name))
+  :set (lambda (sym val)
+         (set sym val)
+         (xwem-set-face-font
+          'xwem-tabber-face
+          (or val (xwem-face-font 'xwem-tabber-face '(default)))
+          '(frame-nonselected tab-selected)))
+  :initialize 'custom-initialize-default
+  :group 'xwem-tab)
+
+(defcustom xwem-tabber-font:frame.nonselected-tab.nonselected nil
+  "Font to be used in selected tab of selected frame."
+  :type '(restricted-sexp :match-alternatives (nil try-font-name))
+  :set (lambda (sym val)
+         (set sym val)
+         (xwem-set-face-font
+          'xwem-tabber-face
+          (or val (xwem-face-font 'xwem-tabber-face '(default)))
+          '(frame-nonselected tab-nonselected)))
+  :initialize 'custom-initialize-default
+  :group 'xwem-tab)
+  
 ;;; Internal variables
 
 (defvar xwem-tabber-map
@@ -164,12 +213,12 @@
 
 ;;; Tabber
 (defstruct xwem-tabber
-  frame					; xwem-frame, our parent
+  frame                                 ; xwem-frame, our parent
   xgeom                                 ; Tabber's geometry
   clients                               ; clients list
 
-  xwin					; Tabber's X-Win
-  xpreparer				; Tabber's X-Pixmap to prepare tab items
+  xwin                                  ; Tabber's X-Win
+  xpreparer                             ; Tabber's X-Pixmap to prepare tab items
   xpix-copy                             ; Tabber's X-Pixmap used to copy to xwin
   
   ;; Margins
@@ -219,14 +268,16 @@
 ;;; Functions
 
 (defsubst xwem-tabber-put-prop (tabber prop val)
-  (setf (xwem-tabber-plist tabber) (plist-put (xwem-tabber-plist tabber) prop val)))
+  (setf (xwem-tabber-plist tabber)
+        (plist-put (xwem-tabber-plist tabber) prop val)))
 (put 'xwem-tabber-put-prop 'lisp-indent-function 2)
 
 (defsubst xwem-tabber-get-prop (tabber prop)
   (plist-get (xwem-tabber-plist tabber) prop))
 
 (defsubst xwem-tabber-rm-prop (tabber prop)
-  (setf (xwem-tabber-plist tabber) (plist-remprop (xwem-tabber-plist tabber) prop)))
+  (setf (xwem-tabber-plist tabber)
+        (plist-remprop (xwem-tabber-plist tabber) prop)))
 
 (defun xwem-tabber-init ()
   "Initialize tabbing."
@@ -293,14 +344,18 @@ command event."
         (X-Event-xmotion-root-y xev)))
       (:X-ButtonRelease
        (xwem-dispatch-command-xevent xev)))))
+(put 'xwem-tabber-smart-drag-frame 'xwem-frame-command t)
 
 ;;;###autoload(autoload 'xwem-tabber-popup-cl-menu "xwem-tabber" "" t)
 (define-xwem-command xwem-tabber-popup-cl-menu ()
   "Popup clients menu."
   (xwem-interactive)
 
-  (when (xwem-cl-alive-p xwem-tabber-click-cl)
-    (xwem-popup-menu (xwem-generate-cl-menu xwem-tabber-click-cl))))
+  (if (xwem-cl-alive-p xwem-tabber-click-cl)
+      (xwem-popup-menu (xwem-generate-cl-menu xwem-tabber-click-cl))
+    ;; TODO
+;    (xwem-popup-menu nil
+     ))
 
 (defun xwem-tabber-cl-at (tabber x y)
   "Return client of TABBER which rectangle covers point at X Y."
@@ -329,7 +384,7 @@ command event."
   (let* ((twidth (X-Geom-width (xwem-tabber-xgeom tabber)))
          (theight (X-Geom-height (xwem-tabber-xgeom tabber)))
          (clients (xwem-tabber-clients tabber))
-	 (clsn (length clients ))       ; number of clients
+         (clsn (length clients ))       ; number of clients
          (off 0)
          tiw twrem rect)
 
@@ -342,13 +397,13 @@ command event."
         (unless (xwem-cl-tab-rect (car clients))
           (setf (xwem-cl-tab-rect (car clients))
                 (make-X-Rect :x 0 :y 0 :width 0 :height 0)))
-	(setq rect (xwem-cl-tab-rect (car clients)))
-	(setf (X-Rect-x rect) off)
-	(setf (X-Rect-width rect) (+ tiw (if (cdr clients) 0 twrem)))
-	(setf (X-Rect-height rect) theight)
+        (setq rect (xwem-cl-tab-rect (car clients)))
+        (setf (X-Rect-x rect) off)
+        (setf (X-Rect-width rect) (+ tiw (if (cdr clients) 0 twrem)))
+        (setf (X-Rect-height rect) theight)
 
         (setq off (+ off (X-Rect-width rect)))
-	(setq clients (cdr clients))))))
+        (setq clients (cdr clients))))))
 
 (defsubst xwem-tabber-safe-regeom (tabber &optional draw-p)
   "Saf variant of `xwem-tabber-regeom'."
@@ -382,7 +437,7 @@ command event."
 (defun xwem-tabber-rect->xpix-rect (tabber rect)
   (xwem-tabber-xpix-rect->rect tabber rect))
 
-(define-xwem-deffered xwem-tabber-redraw xwem-tabber-redraw-1 (tabber &optional  x y width height)
+(define-xwem-deffered xwem-tabber-redraw (tabber &optional  x y width height)
   "Redraw TABBER's rectangle specified by X, Y, WIDTH and HEIGHT.
 If one of optional arguments ommited, full redraw."
   (when (xwem-tabber-p tabber)
@@ -393,15 +448,16 @@ If one of optional arguments ommited, full redraw."
            (height (or height (X-Geom-height xgeom)))
            (tl (xwem-frame-property (xwem-tabber-frame tabber) 'title-layout)))
       (cond ((memq tl '(top bottom))
-             (XCopyArea (xwem-dpy) (xwem-tabber-xpix-copy tabber) (xwem-tabber-xwin tabber)
+             (XCopyArea (xwem-dpy) (xwem-tabber-xpix-copy tabber)
+                        (xwem-tabber-xwin tabber)
                         (XDefaultGC (xwem-dpy)) x y width height x y))
             ((memq tl '(left right))
              (let* ((ximg (XGetImage (xwem-dpy) (xwem-tabber-xpix-copy tabber)
                                      x y width height X-AllPlanes X-ZPixmap))
-                    (rxd (xwem-misc-rotate-data (nth 4 ximg) width height
-                                                (XGetDepth (xwem-dpy)
-                                                           (xwem-tabber-xpix-copy tabber))
-                                                (if (eq tl 'left) 'left 'right)))
+                    (rxd (xwem-misc-rotate-data
+                          (nth 4 ximg) width height
+                          (XGetDepth (xwem-dpy) (xwem-tabber-xpix-copy tabber))
+                          (if (eq tl 'left) 'left 'right)))
                     (dst-height width)
                     (dst-width height)
                     dst-x dst-y)
@@ -413,9 +469,10 @@ If one of optional arguments ommited, full redraw."
                (XPutImage (xwem-dpy) (xwem-tabber-xwin tabber)
                           (XDefaultGC (xwem-dpy))
                           (XGetDepth (xwem-dpy) (xwem-tabber-xpix-copy tabber))
-                          dst-width dst-height dst-x dst-y nil X-ZPixmap rxd)))))))
+                          dst-width dst-height
+                          dst-x dst-y nil X-ZPixmap rxd)))))))
   
-(define-xwem-deffered xwem-tabber-redraw-xrect xwem-tabber-redraw-xrect-1 (tabber &optional xrect)
+(define-xwem-deffered xwem-tabber-redraw-xrect (tabber &optional xrect)
   "Redraw part of TABBER.
 XRECT specifies geometry to redraw.
 Defaultly full redraw."
@@ -429,227 +486,252 @@ Defaultly full redraw."
 
 (defsubst xwem-tabber-frame-win-clients (tabber)
   "Return list of managed clients in TABBER's window."
-  (delq nil (mapcar (lambda (cl)
-                      (and (xwem-cl-managed-p cl '(active inactive)) cl))
-                    (xwem-win-clients (xwem-frame-selwin (xwem-tabber-frame tabber))))))
+  (delq nil (mapcar #'(lambda (cl)
+                        (and (xwem-cl-managed-p cl '(active inactive)) cl))
+                    (xwem-win-clients
+                     (xwem-frame-selwin (xwem-tabber-frame tabber))))))
 
 (defsubst xwem-tabber-clients-equal (cls1 cls2)
   "Return non-nil if each element of CLS1 and CLS2 is `eq'."
   (and (= (length cls1) (length cls2))
        (not (memq nil (or (mapcar* 'eq cls1 cls2))))))
 
-(define-xwem-deffered xwem-tabber-draw-format xwem-tabber-draw-format-1 (cl &optional tabber force-update)
+(define-xwem-deffered xwem-tabber-draw-format (cl &optional tabber force-update)
   "Draw CL's tab.
 If FORCE-UPDATE is non-nil also copy to TABBER x window."
   (when (or (null cl) (xwem-cl-alive-p cl))
     ;; Either empty or valid client
-  (let* ((tabber (or tabber (xwem-cl-tabber cl)))
-         (rect (or (and (xwem-cl-p cl) (xwem-cl-tab-rect cl))
-                   (let ((xgeom (xwem-tabber-xgeom tabber)))
-                     (make-X-Rect :x 0 :y 0 :width (X-Geom-width xgeom)
-                                  :height (X-Geom-height xgeom)))))
-	 (fmt (or (and (xwem-cl-p cl)
-                       (xwem-client-property cl 'xwem-tab-format))
-                  xwem-tab-empty-format))
-	 (xprep (xwem-tabber-xpreparer tabber))
-         (xpcop (xwem-tabber-xpix-copy tabber))
-	 (xoff (X-Rect-x rect))
-	 (yoff (X-Rect-y rect))
-         tag-set currgc fi item fmt-index sfg)
+    (let* ((tabber (or tabber (xwem-cl-tabber cl)))
+           (rect (or (and (xwem-cl-p cl) (xwem-cl-tab-rect cl))
+                     (let ((xgeom (xwem-tabber-xgeom tabber)))
+                       (make-X-Rect :x 0 :y 0 :width (X-Geom-width xgeom)
+                                    :height (X-Geom-height xgeom)))))
+           (fmt (or (and (xwem-cl-p cl)
+                         (xwem-client-property cl 'xwem-tab-format))
+                    xwem-tab-empty-format))
+           (xprep (xwem-tabber-xpreparer tabber))
+           (xpcop (xwem-tabber-xpix-copy tabber))
+           (xoff (X-Rect-x rect))
+           (yoff (X-Rect-y rect))
+           tag-set currgc fi item fmt-index sfg)
  
-    ;; Setup TAG-SET
-    (if (xwem-frame-selected-p (xwem-tabber-frame tabber))
+      ;; Setup TAG-SET
+      (if (xwem-frame-selected-p (xwem-tabber-frame tabber))
+          (if (xwem-cl-p cl)
+              (if (xwem-win-cl-current-p cl)
+                  (setq tag-set (list 'frame-selected 'tab-selected))
+                (setq tag-set (list 'frame-selected 'tab-nonselected)))
+
+            ;; Empty tab item
+            (setq tag-set (list 'frame-selected 'tab-selected)))
+
         (if (xwem-cl-p cl)
             (if (xwem-win-cl-current-p cl)
-                (setq tag-set (list 'frame-selected 'tab-selected))
-              (setq tag-set (list 'frame-selected 'tab-nonselected)))
+                (setq tag-set (list 'frame-nonselected 'tab-selected))
+              (setq tag-set (list 'frame-nonselected 'tab-nonselected)))
 
           ;; Empty tab item
-          (setq tag-set (list 'frame-selected 'tab-selected)))
+          (setq tag-set (list 'frame-nonselected 'tab-selected))))
 
-      (if (xwem-cl-p cl)
-          (if (xwem-win-cl-current-p cl)
-              (setq tag-set (list 'frame-nonselected 'tab-selected))
-            (setq tag-set (list 'frame-nonselected 'tab-nonselected)))
+      ;; Setup currgc, xprep, tabxwin
+      (setq currgc (xwem-face-get-gc (xwem-cl-tab-face cl) tag-set cl))
+      (XSetClipRectangles (xwem-dpy) currgc 0 0 (list rect))
 
-        ;; Empty tab item
-        (setq tag-set (list 'frame-nonselected 'tab-selected))))
-
-    ;; Setup currgc, xprep, tabxwin
-    (setq currgc (xwem-face-get-gc (xwem-cl-tab-face cl) tag-set cl))
-    (XSetClipRectangles (xwem-dpy) currgc 0 0 (list rect))
-
-    (setq sfg (X-Gc-foreground currgc))
-    (xwem-unwind-protect
-        (progn
-          (setf (X-Gc-foreground currgc) (X-Gc-background currgc))
-          (XChangeGC (xwem-dpy) currgc)
-          (XFillRectangles (xwem-dpy) xprep currgc (list rect))
-          (XFillRectangles (xwem-dpy) xpcop currgc (list rect)))
-      (setf (X-Gc-foreground currgc) sfg)
-      (XChangeGC (xwem-dpy) currgc))
-
-    ;; Process format string
-    (setq fmt-index 0)
-    (while (and (< xoff (+ (X-Rect-x rect) (X-Rect-width rect)))
-                (< fmt-index (length fmt)))
-
-      ;; Extract ITEM
-      (setq fi (aref fmt fmt-index))
-      (incf fmt-index)
-      (if (eq fi ?%)
+      (setq sfg (X-Gc-foreground currgc))
+      (xwem-unwind-protect
           (progn
-            (setq fi (aref fmt fmt-index))
-            (setq item (cond ((= fi ?n) (xwem-client-name cl))
-                             ((= fi ?c) (car (xwem-hints-wm-class (xwem-cl-hints cl))))
-                             ((= fi ?C) (cdr (xwem-hints-wm-class (xwem-cl-hints cl))))
-                             ((= fi ?i) (xwem-icons-cl-icon cl (and (not (xwem-frame-selected-p (xwem-cl-frame cl)))
-                                                                    '(shade))))
-                             ((= fi ?s) (xwem-cl-get-psize cl))
-                             ((= fi ?S) (xwem-cl-get-usize cl))
-                             ((= fi ?u) (xwem-cl-get-uptime cl))
-                             ((= fi ?U) (xwem-cl-get-uptime cl))
-                             ((= fi ?f) (int-to-string (xwem-frame-num (xwem-cl-frame cl))))
-                             ((= fi ?F) (xwem-frame-name (xwem-cl-frame cl)))
-                             ((= fi ?*) (if (xwem-cl-marked-p cl) "*" "-"))
-                             ((= fi ?#) (if (XWMProtocol-set-p (xwem-dpy)
-                                                               (xwem-hints-wm-protocols (xwem-cl-hints cl))
-                                                               "WM_DELETE_WINDOW")
-                                            "#" "-"))
-                             ((= fi ?I) (let ((ip (X-WMHints-input-p (xwem-hints-wm-hints (xwem-cl-hints cl))))
-                                              (tf (XWMProtocol-set-p (xwem-dpy)
-                                                                     (xwem-hints-wm-protocols (xwem-cl-hints cl))
-                                                                     "WM_TAKE_FOCUS")))
-                                          (cond ((and ip tf) "L")
-                                                (ip "P")
-                                                (tf "G")
-                                                (t "-"))))
-                             ((= fi ?%) "%")
+            (setf (X-Gc-foreground currgc) (X-Gc-background currgc))
+            (XChangeGC (xwem-dpy) currgc)
+            (XFillRectangles (xwem-dpy) xprep currgc (list rect))
+            (XFillRectangles (xwem-dpy) xpcop currgc (list rect)))
+        (setf (X-Gc-foreground currgc) sfg)
+        (XChangeGC (xwem-dpy) currgc))
 
-                             ;; Emacs lisp
-                             ((= fi ?{)
-                              (let ((substr (substring fmt (1+ fmt-index)))
-                                    elstr)
-                                (unless (string-match "\\(\\([^%]\\|%[^}]\\)*\\)%}" substr)
-                                  (signal 'search-failed fmt "%}"))
-				
-                                ;; extract lisp code and update fmt indexer
-                                (setq elstr (match-string 1 substr))
-                                (incf fmt-index (match-end 0))
+      ;; Process format string
+      (setq fmt-index 0)
+      (while (and (< xoff (+ (X-Rect-x rect) (X-Rect-width rect)))
+                  (< fmt-index (length fmt)))
 
-                                ;; Now time to run emacs lisp.
-				
-                                ;; NOTE:
-                                ;;
-                                ;;  - Due to dynamic scoping, emacs
-                                ;;    lisp code that is in ELSTR can
-                                ;;    access any locally bounded
-                                ;;    variable for example `cl'.
-                                ;;
-                                ;; - It should return string, cons
-                                ;;   cell(image) or nil.
-                                (eval (read elstr))))
-				
-                             ;; Ditig is number of aditional GC
-                             ((and (> (Xforcenum fi) 47) (< (Xforcenum fi) 57))
-                              (let* ((n (string-to-int (char-to-string fi)))
-                                     (gc (xwem-face-get-gc (if (zerop n)
-                                                               (xwem-cl-tab-face cl)
-                                                             (intern-soft (concat "xwem-tabber-face" (int-to-string n))))
-                                           tag-set cl)))
-                                (when (X-Gc-p gc)
-                                  (setq currgc gc)
-                                  (XSetClipRectangles (xwem-dpy) currgc 0 0 (list rect))))
-                              'skip)
+        ;; Extract ITEM
+        (setq fi (aref fmt fmt-index))
+        (incf fmt-index)
+        (if (eq fi ?%)
+            (progn
+              (setq fi (aref fmt fmt-index))
+              (setq item
+                    (cond ((= fi ?n) (xwem-client-name cl))
+                          ((= fi ?c) (car (xwem-hints-wm-class
+                                           (xwem-cl-hints cl))))
+                          ((= fi ?C) (cdr (xwem-hints-wm-class
+                                           (xwem-cl-hints cl))))
+                          ((= fi ?i) (xwem-icons-cl-icon
+                                      cl (and (not (xwem-frame-selected-p
+                                                    (xwem-cl-frame cl)))
+                                              '(shade))))
+                          ((= fi ?s) (xwem-cl-get-psize cl))
+                          ((= fi ?S) (xwem-cl-get-usize cl))
+                          ((= fi ?u) (xwem-cl-get-uptime cl))
+                          ((= fi ?U) (xwem-cl-get-uptime cl))
+                          ((= fi ?f) (int-to-string
+                                      (xwem-frame-num (xwem-cl-frame cl))))
+                          ((= fi ?F) (xwem-frame-name (xwem-cl-frame cl)))
+                          ((= fi ?*) (if (xwem-cl-marked-p cl) "*" "-"))
+                          ((= fi ?#) (if (XWMProtocol-set-p
+                                          (xwem-dpy)
+                                          (xwem-hints-wm-protocols
+                                           (xwem-cl-hints cl))
+                                          "WM_DELETE_WINDOW")
+                                         "#" "-"))
+                          ((= fi ?I) (let ((ip (X-WMHints-input-p
+                                                (xwem-hints-wm-hints
+                                                 (xwem-cl-hints cl))))
+                                           (tf (XWMProtocol-set-p
+                                                (xwem-dpy)
+                                                (xwem-hints-wm-protocols
+                                                 (xwem-cl-hints cl))
+                                                "WM_TAKE_FOCUS")))
+                                       (cond ((and ip tf) "L")
+                                             (ip "P")
+                                             (tf "G")
+                                             (t "-"))))
+                          ((= fi ?%) "%")
 
-                             (t (error 'xwem-error "Unknown token in tabi format"))))
-            ;; size fix
-            (when (and (consp item)
-                       (numberp (car item))
-                       (numberp (cdr item)))
-              (setq item (concat (int-to-string (car item))
-                                 "x"
-                                 (int-to-string (cdr item)))))
-            (incf fmt-index))
+                          ;; Emacs lisp
+                          ((= fi ?{)
+                           (let ((substr (substring fmt (1+ fmt-index)))
+                                 elstr)
+                             (unless (string-match
+                                      "\\(\\([^%]\\|%[^}]\\)*\\)%}" substr)
+                               (signal 'search-failed fmt "%}"))
+                                
+                             ;; extract lisp code and update fmt indexer
+                             (setq elstr (match-string 1 substr))
+                             (incf fmt-index (match-end 0))
 
-        ;; Not %
-        (setq item (char-to-string fi))
-        (while (and (< fmt-index (length fmt))
-                    (not (= (aref fmt fmt-index) ?%)))
-          (setq item (concat item (char-to-string (aref fmt fmt-index))))
-          (incf fmt-index)))
+                             ;; Now time to run emacs lisp.
+                                
+                             ;; NOTE:
+                             ;;
+                             ;;  - Due to dynamic scoping, emacs
+                             ;;    lisp code that is in ELSTR can
+                             ;;    access any locally bounded
+                             ;;    variable for example `cl'.
+                             ;;
+                             ;; - It should return string, cons
+                             ;;   cell(image) or nil.
+                             (eval (read elstr))))
+                                
+                          ;; Ditig is number of aditional GC
+                          ((and (> (Xforcenum fi) 47) (< (Xforcenum fi) 57))
+                           (let* ((n (string-to-int (char-to-string fi)))
+                                  (gc (xwem-face-get-gc
+                                          (if (zerop n)
+                                              (xwem-cl-tab-face cl)
+                                            (intern-soft
+                                             (concat "xwem-tabber-face"
+                                                     (int-to-string n))))
+                                        tag-set cl)))
+                             (when (X-Gc-p gc)
+                               (setq currgc gc)
+                               (XSetClipRectangles (xwem-dpy) currgc
+                                                   0 0 (list rect))))
+                           'skip)
 
-      ;; Display ITEM
-      (cond ((stringp item)
-             ;; Draw text
-             (let* ((font (X-Gc-font currgc))
-                    (ta (X-Text-ascent (xwem-dpy) font item))
-                    (td (X-Text-descent (xwem-dpy) font item))
-                    (hei (X-Rect-height rect))
-                    (ty (+ yoff (/ (- hei (+ ta td)) 2) ta)))
+                          (t (error 'xwem-error "Unknown token in tabi format"))))
+              ;; size fix
+              (when (and (consp item)
+                         (numberp (car item))
+                         (numberp (cdr item)))
+                (setq item (concat (int-to-string (car item))
+                                   "x"
+                                   (int-to-string (cdr item)))))
+              (incf fmt-index))
 
-               (XDrawString (xwem-dpy) xprep currgc xoff ty item)
-               (setq xoff (+ xoff (X-Text-width (xwem-dpy) (X-Gc-font currgc) item)))
-               ))
+          ;; Not %
+          (setq item (char-to-string fi))
+          (while (and (< fmt-index (length fmt))
+                      (not (= (aref fmt fmt-index) ?%)))
+            (setq item (concat item (char-to-string (aref fmt fmt-index))))
+            (incf fmt-index)))
 
-            ((and (consp item)
-                  (X-Pixmap-p (car item))
-                  (X-Pixmap-p (cdr item)))
-             ;; Draw icon
-             (let ((ty (/ (- (X-Rect-height rect) (X-Pixmap-height (car item))) 2)))
-;		   (ximg-mask (X-Pixmap-get-prop (cdr item) 'ximg)))
-               (xwem-unwind-protect
-                   (progn
-                     (setf (X-Gc-clip-mask currgc) (cdr item))
-                     (setf (X-Gc-clip-x-origin currgc) xoff)
-                     (setf (X-Gc-clip-y-origin currgc) (+ yoff ty))
-                     (XChangeGC (xwem-dpy) currgc)
+        ;; Display ITEM
+        (cond ((stringp item)
+               ;; Draw text
+               (let* ((font (X-Gc-font currgc))
+                      (ta (X-Text-ascent (xwem-dpy) font item))
+                      (td (X-Text-descent (xwem-dpy) font item))
+                      (hei (X-Rect-height rect))
+                      (ty (+ yoff (/ (- hei (+ ta td)) 2) ta)))
 
-                     (XCopyArea (xwem-dpy) (car item) xprep currgc 0 0
-                                (X-Pixmap-width (car item)) (X-Pixmap-height (car item))
-                                xoff (+ yoff ty)))
-                 (setf (X-Gc-clip-mask currgc) X-None)
-                 (setf (X-Gc-clip-x-origin currgc) 0)
-                 (setf (X-Gc-clip-y-origin currgc) 0)
-                 (XChangeGC (xwem-dpy) currgc)
-                 (XSetClipRectangles (xwem-dpy) currgc 0 0 (list rect)))
+                 (XDrawString (xwem-dpy) xprep currgc xoff ty item)
+                 (setq xoff (+ xoff (X-Text-width
+                                     (xwem-dpy) (X-Gc-font currgc) item)))
+                 ))
 
-               (setq xoff (+ xoff (X-Pixmap-width (car item))))))
+              ((and (consp item)
+                    (X-Pixmap-p (car item))
+                    (X-Pixmap-p (cdr item)))
+               ;; Draw icon
+               (let ((ty (/ (- (X-Rect-height rect)
+                               (X-Pixmap-height (car item))) 2)))
+;                  (ximg-mask (X-Pixmap-get-prop (cdr item) 'ximg)))
+                 (xwem-unwind-protect
+                     (progn
+                       (setf (X-Gc-clip-mask currgc) (cdr item))
+                       (setf (X-Gc-clip-x-origin currgc) xoff)
+                       (setf (X-Gc-clip-y-origin currgc) (+ yoff ty))
+                       (XChangeGC (xwem-dpy) currgc)
 
-            ((or (null item)
-                 (eq item 'skip)) nil)
+                       (XCopyArea (xwem-dpy) (car item) xprep currgc 0 0
+                                  (X-Pixmap-width (car item))
+                                  (X-Pixmap-height (car item))
+                                  xoff (+ yoff ty)))
+                   (setf (X-Gc-clip-mask currgc) X-None)
+                   (setf (X-Gc-clip-x-origin currgc) 0)
+                   (setf (X-Gc-clip-y-origin currgc) 0)
+                   (XChangeGC (xwem-dpy) currgc)
+                   (XSetClipRectangles (xwem-dpy) currgc 0 0 (list rect)))
 
-            (t (error 'xwem-error "Unknown Item" item)))
-      )                                 ;  while
+                 (setq xoff (+ xoff (X-Pixmap-width (car item))))))
 
-    ;; Compose xpreparer
-    (when (> xoff (+ (X-Rect-x rect) (X-Rect-width rect)))
-      (setq xoff (+ (X-Rect-x rect) (X-Rect-width rect))))
+              ((or (null item)
+                   (eq item 'skip)) nil)
+
+              (t (error 'xwem-error "Unknown Item" item)))
+        )                               ;  while
+
+      ;; Compose xpreparer
+      (when (> xoff (+ (X-Rect-x rect) (X-Rect-width rect)))
+        (setq xoff (+ (X-Rect-x rect) (X-Rect-width rect))))
     
-    (XCopyArea (xwem-dpy) xprep xpcop currgc
-               (X-Rect-x rect) (X-Rect-y rect) (X-Rect-width rect) (X-Rect-height rect)
-               (+ (X-Rect-x rect)
-                  (/ (- (+ (X-Rect-x rect) (X-Rect-width rect)) xoff) 2))
-               (X-Rect-y rect))
-
-    (let ((ldgc (xwem-face-get-gc (xwem-cl-tab-face cl) (cons 'delimiter-left tag-set) cl))
-          (rdgc (xwem-face-get-gc (xwem-cl-tab-face cl) (cons 'delimiter-right tag-set) cl)))
-      (XDrawLine (xwem-dpy) xpcop ldgc
+      (XCopyArea (xwem-dpy) xprep xpcop currgc
                  (X-Rect-x rect) (X-Rect-y rect)
-                 (X-Rect-x rect) (+ (X-Rect-y rect) (X-Rect-height rect)))
-      (XDrawLine (xwem-dpy) xpcop rdgc
-                 (+ -1 (X-Rect-x rect) (X-Rect-width rect)) (X-Rect-y rect)
-                 (+ -1 (X-Rect-x rect) (X-Rect-width rect)) (+ (X-Rect-y rect) (X-Rect-height rect)))
-      )
+                 (X-Rect-width rect) (X-Rect-height rect)
+                 (+ (X-Rect-x rect)
+                    (/ (- (+ (X-Rect-x rect) (X-Rect-width rect)) xoff) 2))
+                 (X-Rect-y rect))
 
-    ;; Finally apply change to xwin
-    (when force-update
-      (xwem-tabber-redraw-xrect-1 tabber rect))
+      (let ((ldgc (xwem-face-get-gc (xwem-cl-tab-face cl)
+                    (cons 'delimiter-left tag-set) cl))
+            (rdgc (xwem-face-get-gc (xwem-cl-tab-face cl)
+                    (cons 'delimiter-right tag-set) cl)))
+        (XDrawLine (xwem-dpy) xpcop ldgc
+                   (X-Rect-x rect) (X-Rect-y rect)
+                   (X-Rect-x rect) (+ (X-Rect-y rect) (X-Rect-height rect)))
+        (XDrawLine (xwem-dpy) xpcop rdgc
+                   (+ -1 (X-Rect-x rect) (X-Rect-width rect))
+                   (X-Rect-y rect)
+                   (+ -1 (X-Rect-x rect) (X-Rect-width rect))
+                   (+ (X-Rect-y rect) (X-Rect-height rect))))
 
-    ;; Unmark client as need to be redrawed
-    (when (xwem-cl-p cl)
-      (xwem-cl-rem-sys-prop cl 'xwem-tab-need-redraw))
-    )))
+      ;; Finally apply change to xwin
+      (when force-update
+        (xwem-tabber-redraw-xrect-1 tabber rect))
+
+      ;; Unmark client as need to be redrawed
+      (when (xwem-cl-p cl)
+        (xwem-cl-rem-sys-prop cl 'xwem-tab-need-redraw))
+      )))
 
 (defun xwem-tabber-regeom-p (tabber)
   "Return non-nil if TABBER is regeomed."
@@ -661,7 +743,7 @@ If FORCE-UPDATE is non-nil also copy to TABBER x window."
         (xwem-tabber-regeom tabber))
       t)))
 
-(define-xwem-deffered xwem-tabber-draw xwem-tabber-draw-1 (tabber &optional force-draw)
+(define-xwem-deffered xwem-tabber-draw (tabber &optional force-draw)
   "On TABBER draw client's tabs.
 If FORCE-DRAW is non-nil redraw tabber even if it logically does not
 need to be redrawed."
@@ -683,13 +765,17 @@ need to be redrawed."
       (if (or (xwem-tabber-regeom-p tabber) force-draw)
           (setq cls-to-draw (or (xwem-tabber-clients tabber) (list nil))
                 need-draw-p t)          ; all clients
-        (setq cls-to-draw (delq nil (mapcar (lambda (cl)
-                                              (and (xwem-cl-get-sys-prop cl 'xwem-tab-need-redraw) cl))
-                                            (xwem-tabber-clients tabber)))
+        (setq cls-to-draw
+              (delq nil (mapcar #'(lambda (cl)
+                                    (and (xwem-cl-get-sys-prop
+                                          cl 'xwem-tab-need-redraw) cl))
+                                (xwem-tabber-clients tabber)))
               need-draw-p cls-to-draw))
 
       (when need-draw-p
-        (mapc '(lambda (cl) (xwem-tabber-draw-format-1 cl tabber)) cls-to-draw)
+        (mapc #'(lambda (cl)
+                  (xwem-tabber-draw-format-1 cl tabber))
+              cls-to-draw)
         (xwem-tabber-redraw-1 tabber)))))
 
 (defun xwem-tabber-event-handler (xdpy xwin xev)
@@ -697,7 +783,7 @@ need to be redrawed."
   (let ((tabber (X-Win-get-prop xwin 'xwem-tabber)))
     (when (xwem-tabber-p tabber)
       (X-Event-CASE xev
-	(:X-Expose
+        (:X-Expose
          (xwem-tabber-redraw-xrect
           tabber (xwem-tabber-rect->xpix-rect
                   tabber
@@ -706,7 +792,7 @@ need to be redrawed."
                                :width (X-Event-xexpose-width xev)
                                :height (X-Event-xexpose-height xev)))))
 
-	(:X-DestroyNotify
+        (:X-DestroyNotify
          (when (xwem-tabber-p tabber)
            (XFreePixmap (xwem-dpy) (xwem-tabber-xpreparer tabber))
            (XFreePixmap (xwem-dpy) (xwem-tabber-xpix-copy tabber))
@@ -734,21 +820,23 @@ need to be redrawed."
          (tabber (make-xwem-tabber :frame frame
                                    :xgeom xgeom))
          (xdpy (xwem-dpy))
-         (w (XCreateWindow xdpy
-			   (xwem-frame-xwin frame)
-			   (X-Geom-x xgeom) (X-Geom-y xgeom)
-			   (X-Geom-width xgeom) (X-Geom-height xgeom)
-			   (X-Geom-border-width xgeom)
-			   nil nil nil  ;X-InputOutput nil
-			   (make-X-Attr :background-pixel (XWhitePixel (xwem-dpy))
-                                        :bit-gravity X-StaticGravity
-                                        :backing-store X-Always))))
+         (w (XCreateWindow
+             xdpy
+             (xwem-frame-xwin frame)
+             (X-Geom-x xgeom) (X-Geom-y xgeom)
+             (X-Geom-width xgeom) (X-Geom-height xgeom)
+             (X-Geom-border-width xgeom)
+             nil nil nil                ;X-InputOutput nil
+             (make-X-Attr :background-pixel (XWhitePixel (xwem-dpy))
+                          :bit-gravity X-StaticGravity
+                          :backing-store X-Always))))
 
     (setf (xwem-tabber-xwin tabber) w)
     (X-Win-put-prop w 'xwem-tabber tabber)
 
-    (XSelectInput xdpy w (Xmask-or XM-Exposure XM-StructureNotify
-                                   XM-ButtonPress XM-ButtonRelease XM-ButtonMotion))
+    (XSelectInput xdpy w
+                  (Xmask-or XM-Exposure XM-StructureNotify
+                            XM-ButtonPress XM-ButtonRelease XM-ButtonMotion))
     (X-Win-EventHandler-add w 'xwem-tabber-event-handler 0
                             (list X-Expose X-DestroyNotify X-ButtonPress
                                   X-ButtonRelease X-MotionNotify))
@@ -824,16 +912,20 @@ need to be redrawed."
       (when (X-Pixmap-p (xwem-tabber-xpix-copy tabber))
         (XFreePixmap (xwem-dpy) (xwem-tabber-xpix-copy tabber)))
       (setf (xwem-tabber-xpreparer tabber)
-            (XCreatePixmap (xwem-dpy) (make-X-Pixmap :dpy (xwem-dpy)
-                                                     :id (X-Dpy-get-id (xwem-dpy)))
+            (XCreatePixmap (xwem-dpy)
+                           (make-X-Pixmap :dpy (xwem-dpy)
+                                          :id (X-Dpy-get-id (xwem-dpy)))
                            (xwem-tabber-xwin tabber) (XDefaultDepth (xwem-dpy))
                            (X-Geom-width xgeom) (X-Geom-height xgeom)))
       (setf (xwem-tabber-xpix-copy tabber)
-            (XCreatePixmap (xwem-dpy) (make-X-Pixmap :dpy (xwem-dpy)
-                                                     :id (X-Dpy-get-id (xwem-dpy)))
+            (XCreatePixmap (xwem-dpy)
+                           (make-X-Pixmap :dpy (xwem-dpy)
+                                          :id (X-Dpy-get-id (xwem-dpy)))
                            (xwem-tabber-xwin tabber) (XDefaultDepth (xwem-dpy))
-                           (X-Geom-width xgeom) (X-Geom-height xgeom))))
-
+                           (X-Geom-width xgeom) (X-Geom-height xgeom)))
+          (when xwem-misc-turbo-mode
+            (XSetWindowBackgroundPixmap (xwem-dpy) (xwem-tabber-xwin tabber)
+                                        (xwem-tabber-xpix-copy tabber))))
     (xwem-tabber-regeom tabber)))
 
 ;;; Frame Hooks
@@ -898,7 +990,8 @@ need to be redrawed."
     (when (and tabber
                (memq cl (xwem-tabber-clients tabber))
                (xwem-win-selwin-p (xwem-cl-win cl)))
-      (xwem-cl-put-sys-prop cl 'xwem-tab-need-redraw t) ; mark cl as need to be redrawed
+      ;; mark cl as need to be redraw
+      (xwem-cl-put-sys-prop cl 'xwem-tab-need-redraw t)
       (xwem-tabber-draw tabber))))
 
 
